from astropy.io import fits
from astropy.time import Time
from astropy.coordinates import Latitude, Longitude, Angle
from astropy.table import Table, Column
from astropy import units as u
import os, shutil, sys
import datetime
import numpy as np

import modules.utils as utils
#import modules.astrometry as astrometry


def File_Names(directory, lst=None):
    file_name=[]
    if lst is None: # read from directory
        tmp=os.listdir(directory)
        onlyfiles=[f for f in tmp if os.path.isfile(os.path.join(directory,f))]
        for f in onlyfiles:
            f_old=f
            f=f.upper()
           
            if f.endswith('FIT') or f.endswith('FTS') or f.endswith('FITS') :
                file_name.append(os.path.join(directory, f_old))
            if f.endswith('FIT.GZ') or f.endswith('FTS.GZ') or f.endswith('FITS.GZ') :
                file_name.append(os.path.join(directory, f_old)) 
            if f.endswith('FIT.FZ') or f.endswith('FTS.FZ') or f.endswith('FITS.FZ') :
                file_name.append(os.path.join(directory, f_old))  
            
    file_name.sort()
                
    return file_name

def Compressed(name):
    f_extension=os.path.splitext(name)[1].upper()
    if f_extension in ['.FIT','FTS','FITS']:
        compress_type=None
    if f_extension in ['GZ']:
        compress_type='gzip'
    if f_extension in ['FZ']:
        compress_type='fpack'
    
    return compress_type

def ImageType(file_name, header=None):
    
    if header is None:
        hdu=fits.open(file_name)
        header=hdu[0].header
    else:
        header=header
        
    tmp_imagetype_keyword='IMAGETYP'
    try:
        tmp_imtype=header[tmp_imagetype_keyword].strip().lower()
        if tmp_imtype=='':
            imtype='light' 
        if tmp_imtype=='object' or tmp_imtype=='light' or tmp_imtype=='light frame' or tmp_imtype=='reduced_light':
            	imtype='light'
        if tmp_imtype=='flat' or tmp_imtype=='flat frame' or tmp_imtype=='flat-field' or tmp_imtype=='flat field' or tmp_imtype=='flat-field frame' or tmp_imtype=='flat field frame' or tmp_imtype=='flat_field':
            	imtype='flat'
        if tmp_imtype=='dark' or tmp_imtype=='dark frame' or tmp_imtype=='dark-frame' or tmp_imtype=='dark_frame':
            	imtype='dark'
        if tmp_imtype=='bias' or tmp_imtype=='bias frame' or tmp_imtype=='bias-frame' or tmp_imtype=='bias_frame':
            	imtype='bias'
        if tmp_imtype=='masterbias' or tmp_imtype=='masterbias frame' or tmp_imtype=='master bias frame' or tmp_imtype=='master bias' or tmp_imtype=='master_bias':
            imtype='masterbias'
        if tmp_imtype=='masterdark' or tmp_imtype=='masterdark frame' or tmp_imtype=='master dark frame' or tmp_imtype=='master dark' or tmp_imtype=='master_dark' or tmp_imtype=='MasterDark':
            imtype='masterdark'
        if tmp_imtype=='masterflat' or tmp_imtype=='masterflat frame' or tmp_imtype=='master flat frame' or tmp_imtype=='master flat' or tmp_imtype=='master_flat' or tmp_imtype=='MasterFlat':
            imtype='masterflat'
    
    except:
        imtype='light'
    
    
    return imtype
  
def Parameters(name, header=None):
    parameter={}
    
    if header==None:
        hdu=fits.open(name) 
        if len(hdu)==1:
            header=hdu[0].header
            data=hdu[0].data
            tb=1
        elif len(hdu)==2:
            header=hdu[0].header
            data=hdu[0].data
            header_tab=hdu[1].header
            tb_data=hdu[1].data
            tb=2
        elif len(hdu)==3:
            header=hdu[0].header
            data=hdu[0].data
            tb_header=hdu[1].header
            tb_data=hdu[1].data
            tb_header_un=hdu[2].header
            tb_data_un=hdu[2].data
            tb=3			
        else:
            header=hdu[0].header
            tb=1
    else:
        header=header
        
    parameter['DATA_ORIGINAL']=data
    data = data.byteswap(inplace=True).newbyteorder()
    data=data.astype('float64')
    parameter['DATA']=data
    parameter['HEADER']=header
    
    parameter['IMAGE_TYPE']=ImageType(name,header=header)
    
    naxis1=int(header['NAXIS1'])
    naxis2=int(header['NAXIS2'])
		
    wcs_keyword='CD1_1'
    try:
        tmp_wcs=header[wcs_keyword]
        wcs='Y'
    except:
        wcs='N'
			
    if wcs=='Y':
        crval1=header['CRVAL1']
        crval2=header['CRVAL2']
        crpix1=header['CRPIX1']
        crpix2=header['CRPIX2']
        fovX, fovY, pixel_scale=utils.FOV(header=header)
        centralX, centralY=utils.CENTRAL(header=header)
    else:
        crval1=0
        crval2=0
        crpix1=0
        crpix2=0
        fovX=0
        fovY=0
        centralX=0
        centralY=0
        pixel_scale=0
    
    tmp_exposure_keyword='EXPTIME'
    try:
        exposure=header[tmp_exposure_keyword]
    except:
        tmp_exposure_keyword='EXPOSURE'
        try:
            exposure=header[tmp_exposure_keyword]
        except:
            exposure=0
			
    try:
        dateobs=header['DATE-OBS']		
        if len(dateobs.strip())<11:
            timeobs=header['TIME-OBS']
#print(timeobs)
            dateobs=dateobs+'T'+timeobs
        try:
            jd=header['JD']				
        except:
            t = Time(dateobs, format='isot', scale='utc')
            jd=t.jd
    except:
        dateobs=header['DATE-BEG']
        try:
            jd=header['JD']				
        except:
            t = Time(dateobs, format='isot', scale='utc')
            jd=t.jd+exposure/86400
	  
    tmp_binning_keyword='BINNING'			
    try:
        xbinning=header[tmp_binning_keyword]
        ybinning=header[tmp_binning_keyword]				
    except:
        tmp_xbinning_keyword='XBINNING'
        tmp_ybinning_keyword='YBINNING'
        try:			
            xbinning=header[tmp_xbinning_keyword]
            ybinning=header[tmp_ybinning_keyword]			
        except:
            xbinning=1
            ybinning=1
		
    temp_temp_keyword='CCD-TEMP'
    try:
        temp=round(header[temp_temp_keyword])
    except:
        temp=0
            
    temp_filter_keyword='FILTER'
    try:
        imfilter=header[temp_filter_keyword]
    except:
        imfilter='None'	
		
    temp_telescope_keyword='TELESCOP'
    try:
        telescope=header[temp_telescope_keyword]
    except:
        telescope=''
    temp_instrument_keyword='INSTRUME'
    try:
        instrument=header[temp_instrument_keyword].strip()
    except:
        temp_instrument_keyword='DEVICE'
        try:
            instrument=header[temp_instrument_keyword].strip()
        except:
            instrument=''
    temp_observatory_keyword='OBSERVAT'
    try:
        observatory=header[temp_observatory_keyword].strip()
    except:
        temp_observatory_keyword='SITE'
    try:
        observatory=header[temp_observatory_keyword].strip()
    except:
        observatory='Kolonica'
				
    temp_obslat_keyword='SITELAT'
    try:
        obslat=header[temp_obslat_keyword].strip()
        tmp=obslat.split()
        obslat=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
    except:
        temp_obslat_keyword='OBS-LAT'
        try:
            obslat=header[temp_obslat_keyword].strip()
            tmp=obslat.split()
            obslat=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
        except:
            if observatory=='Kolonica':
                obslat=+48.934917
            else:
                obslat=0.0
		
    temp_obslong_keyword='SITELONG'
    try:
        obslon=header[temp_obslong_keyword].strip()
        tmp=obslon.split()
        obslon=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
			
    except:
        temp_obslong_keyword='OBS-LON'
        try:
            obslon=header[temp_obslong_keyword].strip()
            tmp=obslon.split()
            obslon=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
				
        except:
            if observatory=='Kolonica':
                obslon=22.2738
            else:
                obslon=0.0
				
    temp_object_keyword='OBJECT'
    try:
        temp_object=header[temp_object_keyword].strip()
        if temp_object==' ' or temp_object=='' :
            temp_object='None'
    except:
        temp_object='None'
		
    temp_frame_keyword='FRAME'
    try:
        temp_frame=header[temp_frame_keyword].strip()
        if temp_frame==' ' or temp_frame=='' :
            temp_frame=''
    except:
        temp_frame=''
    temp_airmass_keyword='AIRMASS'
    try:
        temp_airmass=header[temp_airmass_keyword].strip()
        if temp_airmass==' ' or temp_airmass=='' :
            temp_airmass=''
    except:
        temp_airmass=''
    temp_gain_keyword='GAIN'
    try:
        temp_gain=header[temp_gain_keyword].strip()
        if temp_gain=='' or temp_gain==' ':
            temp_gain=0
    except:
        temp_gain=0
    
		# reading table data if exist
    work_name=dateobs+'_'+imfilter.replace(" ","")+'_'+temp_object.replace(" ","")+'.fit'
                   
    parameter['ORIGINAL_NAME']=name
    parameter['WORKING_NAME']=name 
    parameter['TMP_NAME']=work_name
    parameter['WCS']=wcs
    parameter['WCS_NAME']=name 
    parameter['CRVAL1']=crval1
    parameter['CRVAL2']=crval2
    parameter['CRPIX1']=crpix1
    parameter['CRPIX2']=crpix2
    parameter['CENTER1']=centralX
    parameter['CENTER2']=centralY
    parameter['FOV1']=fovX
    parameter['FOV2']=fovY
    parameter['PIXSCALE']=pixel_scale
    parameter['DATEOBS']=dateobs
    parameter['JD']=jd
    parameter['BJD']=0
    parameter['BAR_CORR']=0
    parameter['NAXIS1']=naxis1
    parameter['NAXIS2']=naxis2
    parameter['XBINNING']=xbinning
    parameter['YBINNING']=ybinning
    parameter['TEMPERATURE']=temp
    parameter['FILTER']=imfilter
    parameter['EXPTIME']=exposure
    parameter['TELESCOPE']=telescope
    parameter['CAMERA']=instrument
    parameter['OBSERVATORY']=observatory
    parameter['OBSLAT']=obslat
    parameter['OBSLON']=obslon
    parameter['OBJECT']=temp_object
    parameter['FRAME']=temp_frame
    parameter['AIRMASS']=temp_airmass
    parameter['GAIN']=temp_gain
    
    if header is None:
        hdu.close()
		
    if tb==1:			
        parameter['Table']=''
        parameter['Table_Un']=''
    elif tb==2:
        tab=Table.read(file_name, hdu=1)
        parameter['Table']=tab
        parameter['Table_Un']=''
    elif tb==3:
        tab=Table.read(file_name, hdu=1)
        parameter['Table']=tab
        tab=Table.read(file_name, hdu=2)
        parameter['Table_Un']=tab

    return parameter    

def Save_To_FITS(save_name, table, image):
    table.rename_column('recno','APASS')
    
    hdu=fits.open(image['ORIGINAL_NAME']) 
    col1 = fits.Column(name='USNO-B1.0', format='15A', array=table['USNO-B1.0'])
    col2 = fits.Column(name='APASS', format='15A', array=table['APASS'])
    col3 = fits.Column(name='VSX', format='15A', array=table['Name'])
    col4 = fits.Column(name='RAJ2000', format='E', array=table['RAJ2000'])
    col5 = fits.Column(name='DEJ2000', format='E', array=table['DEJ2000'])
    col6 = fits.Column(name='CALIBRATED', format='E', array=table['CALIBRATED'])
    col7 = fits.Column(name='CALIBRATED_ERR', format='E', array=table['CALIBRATED_ERR'])
    col8 = fits.Column(name='INSTRUMENTAL', format='E', array=table['MAG_AUTO'])
    col9 = fits.Column(name='INSTRUMENTAL_ERR', format='E', array=table['MAGERR_AUTO'])
    
    cols = fits.ColDefs([col1, col2, col3,col4, col5,col6,col7,col8,col9])
                         #col4, col5, col6, col7, col8, col9, col22, col23,col24, col25, col26, col27, col28 ])
# 		       # , ,  ])
    tbhdu = fits.BinTableHDU.from_columns(cols, name='Photometry-Matched')
    tbhdu.header['ZERPOINT']=table.meta['comments'][0]
    tbhdu.header['ZERPOINTE']=table.meta['comments'][1]
    tbhdu.header['NUM_STARS']=table.meta['comments'][2]
    hdu[0].header['BJD']=image['BJD']
    hdu.append(tbhdu)
    hdu.writeto(save_name,overwrite=True)
    #if os.path.isfile(name):
     #   os.remove(name)
    
    
    return True

def Copy_To_Tmp(name, tmp_dir):
    
    if Compressed(name)=='gzip':
        pass
    elif Compressed(name)=='fpack':
        pass
    else:
        new_name=tmp_dir+os.path.basename(name)
        cp='cp "'+name+'" '+new_name
        os.system(cp)
        
        
    
    
    #new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
	                        
    return new_name



