import sys, os
from astropy.table import Table, hstack, Column
from astropy.coordinates import SkyCoord, EarthLocation, AltAz
from astropy import units as u
from astropy.time import Time
from astropy.io import fits
import numpy as np
#import photutils as phot
import uuid 
from astropy.stats import sigma_clipped_stats,SigmaClip
import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
from astropy.visualization import SqrtStretch, LogStretch, LinearStretch, PowerStretch
from astropy.visualization.mpl_normalize import ImageNormalize
import sep
import modules.utils as utils



class Photometry_SEP():
    def __init__(self, global_parameters, tmp_dir):	
        self.SIGMA_CLIP=float(global_parameters['SIGMA_CLIP'])
        self.MASK=global_parameters['MASK'].upper().strip()
        self.BOX_SIZE= int(global_parameters['BOX_SIZE'])
        self.FILTER_SIZE=int(global_parameters['FILTER_SIZE'])
        self.TRESHOLD=float(global_parameters['TRESHOLD'])
        self.ZERO_POINT=20.0
        
        self.GAIN=float(global_parameters['GAIN'].strip())
        self.MINAREA=float(global_parameters['MINAREA'].strip())
        
            
        
    def Background(self, image):
        if self.MASK=='NONE':
            self.MASK=None
        else:
            self.MASK=None
        data=image['DATA']
        bkg=sep.Background(data, mask=self.MASK, bw=self.BOX_SIZE, bh=self.BOX_SIZE, fw=self.FILTER_SIZE, fh=self.FILTER_SIZE)
       
        return bkg
    
    def Background_Plot(self, background, name, type='PNG', normalize='log', colorbar=True, cmap='Greys_r'):
        if type.upper()=='PNG':
            if normalize.upper()=='LOG':
                norm = ImageNormalize(stretch=LogStretch())
            
            if normalize.upper()=='LINEAR':
                norm = ImageNormalize(stretch=LinearStretch())
                
            if normalize.upper()=='POWER':
                norm = ImageNormalize(stretch=PowerStretch(3))
    
            if normalize.upper()=='SQRT':
                norm = ImageNormalize(stretch=SqrtStretch())
                
            plt.figure(figsize = (15,15))        
            plt.imshow(background.back(),norm=norm,origin='lower', cmap=cmap)
            if colorbar:
                plt.colorbar();
            else:
                pass
            plt.savefig(name)
        else:
            pass

        return True
    
    def Sources(self, image, bkg):
        data=image['DATA']
        data_sub=data-bkg
        if not image['GAIN']==0:
            self.GAIN=image['GAIN']
        #print(self.GAIN)
        sources=sep.extract(data_sub, self.TRESHOLD, gain=self.GAIN, minarea=self.MINAREA,
                            mask=self.MASK, err=bkg.globalrms)
        #print(data)
        #sources=[]
        #print(src)
        return sources
    
    def Sources_Plot(self, sources, data, name, type='PNG', normalize='log', colorbar=True, cmap='Greys_r'):
        if type.upper()=='PNG':
            if normalize.upper()=='LOG':
                norm = ImageNormalize(stretch=LogStretch())
            
            if normalize.upper()=='LINEAR':
                norm = ImageNormalize(stretch=LinearStretch())
                
            if normalize.upper()=='POWER':
                norm = ImageNormalize(stretch=PowerStretch(3))
    
            if normalize.upper()=='SQRT':
                norm = ImageNormalize(stretch=SqrtStretch())
            
            plt.figure(figsize = (15,15))
            plt.imshow(data,norm=norm,origin='lower', cmap=cmap)
            ax=plt.gca()
            for i in range(len(sources)):
                e = Ellipse(xy=(sources['x'][i], sources['y'][i]),
                width=12*sources['a'][i],
                height=12*sources['b'][i],
                angle=sources['theta'][i] * 180. / np.pi)
                e.set_facecolor('none')
                e.set_edgecolor('blue')
                e.set_linewidth(1)
                ax.add_patch(e)

            if colorbar:
                plt.colorbar();
            else:
                pass
            plt.savefig(name)
        else:
            pass
        
        return True
    
    def Photometry(self, sources, image, bkg, aper_type='flux_auto', aper_radius=5.0):
        data=image['DATA']
        data_sub=data-bkg
        x= sources['x']
        y= sources['y']
        a= sources['a']
        b= sources['b']
        theta=sources['theta']
        theta[theta>np.pi/2]=np.pi/2
        
        if aper_type=='flux_auto':
            kronrad, krflag = sep.kron_radius(data_sub, x,y, a,b,theta, 6.0)
            flux, fluxerr, flag = sep.sum_ellipse(data_sub, x,y,a,b,theta ,r=2.5*kronrad)
            flag |= krflag
            r, flag = sep.flux_radius(data_sub, x, y, 6.*a, 0.5, normflux=flux, subpix=5)
            sig = 2. / 2.35 * r  # r from sep.flux_radius() above, with fluxfrac = 0.5
            xwin, ywin, flag = sep.winpos(data_sub, x,y, sig)
            mag=np.full(len(flux),0.)
            magerror=np.full(len(flux),0.)
            #return flux, fluxerr, flag, xwin, ywin
            
        if aper_type=='mag_auto':
            kronrad, krflag = sep.kron_radius(data_sub, x,y, a,b,theta, 6.0)
            flux, fluxerr, flag = sep.sum_ellipse(data_sub, x,y,a,b,theta ,r=2.5*kronrad, err=bkg.globalrms, gain=1.5)
            flag |= krflag
            r, flag = sep.flux_radius(data_sub, x, y, 6.*a, 0.5, normflux=flux, subpix=5)
            sig = 2. / 2.35 * r  # r from sep.flux_radius() above, with fluxfrac = 0.5
            xwin, ywin, flag = sep.winpos(data_sub, x,y, sig)
            mag=-2.5*np.log10(flux)+self.ZERO_POINT
            magerror=2.5/np.log(10)*fluxerr/flux
        
        measurements=utils.Convert_To_Table(xwin,ywin, flux, fluxerr, mag, magerror,flag)
            
        
      
        #print(measurements)
        
        return measurements
        
            #return mag, magerror, flag, xwin, ywin
            
        
            
            
        
        #flux, fluxerr, flag = sep.sum_circle(data_sub, x, y, 3.)
        
       
        
        
    

    
    def Position(self, sources, data, bkg):
        flux, fluxerror, flag=self.Photometry(sources, data, bkg, aper_type='flux_auto')
        
        dd=[]
        return dd
    
        
    
    
        


class Photometry():
    def __init__(self, global_parameters, tmp_dir):	
        self.test=True
        self.BACKGROUND=global_parameters['BACKGROUND'].upper().strip()
        self.SIGMA_CLIP=float(global_parameters['SIGMA_CLIP'])
        self.BOX_SIZE= int(global_parameters['BOX_SIZE'])
        self.FILTER_SIZE=int(global_parameters['FILTER_SIZE'])
        self.MASK=global_parameters['MASK'].upper().strip()
        self.FWHM=float(global_parameters['FWHM'])
        self.TRESHOLD=float(global_parameters['TRESHOLD'])
        
    def Background(self, data):
        #print(data)
        if self.BACKGROUND=='MEAN':
            bkg_estimator=phot.MeanBackground()
            bkgrms_estimator=phot.StdBackgroundRMS()
        elif self.BACKGROUND=='MEDIAN':
            bkg_estimator = phot.MedianBackground()
            bkgrms_estimator=phot.MADStdBackgroundRMS()
        elif self.BACKGROUND=='MODE':
            bkg_estimator=phot.ModeEstimatorBackground()
            bkgrms_estimator=phot.MADStdBackgroundRMS()
        elif self.BACKGROUND=='MMM':
            bkg_estimator=phot.MMMBackground()
            bkgrms_estimator=phot.MADStdBackgroundRMS()
        elif self.BACKGROUND=='SEX':
            bkg_estimator=phot.SExtractorBackground()
            bkgrms_estimator=phot.MADStdBackgroundRMS()
        elif self.BACKGROUND=='BIW':
            bkg_estimator=phot.BiweightLocationBackground()
            bkgrms_estimator=phot.BiweightScaleBackgroundRMS()
        
        if self.MASK=='NONE':
            self.MASK=None
        else:
            pass
        
        #print(self.MASK)
        sigma_clip = SigmaClip(sigma=self.SIGMA_CLIP)        
        bkg = phot.Background2D(data, self.BOX_SIZE , filter_size=self.FILTER_SIZE, mask=self.MASK, sigma_clip=sigma_clip, bkg_estimator=bkg_estimator, bkgrms_estimator=bkgrms_estimator)
        
        
        return bkg
    
    def Background_Plot(self, background, name, type='PNG', mesh=False):
        if type.upper()=='PNG':
            norm = ImageNormalize(stretch=LogStretch())
            plt.figure(figsize = (15,15))
            
            if mesh:
                plt.imshow(background.background,norm=norm,origin='lower', cmap='Greys_r')
                background.plot_meshes(outlines=True, marker='')
                plt.savefig(name)
            else:
                plt.imshow(background.background,norm=norm,origin='lower', cmap='Greys_r')
                plt.savefig(name)
            
        return True
    
      
    
    
    def Sources(self, data):
        #print(data)
        mean, median, std = sigma_clipped_stats(data, sigma=self.SIGMA_CLIP) 
        daofind = phot.DAOStarFinder(threshold=self.TRESHOLD*std,fwhm=self.FWHM, ) 
        sources= daofind(data)
        return sources
    
    def Sources_Plot(self, sources, data,name,aper=3.0, type='PNG'):
        # aper - circuular aperture for display
        positions = np.transpose((sources['xcentroid'], sources['ycentroid']))
        apertures = phot.CircularAperture(positions, r=aper)
        if type.upper()=='PNG':            
            norm = ImageNormalize(stretch=LogStretch())
            plt.figure(figsize = (15,15))
            plt.imshow(data, cmap='Greys', origin='lower', norm=norm)
            apertures.plot(color='blue', lw=1.5, alpha=0.5)
            plt.savefig(name)
            
        
        return True
    
    def Sources_Save(self, sources, name):
        sources.write(name, format='ascii')
        
        return True
    
    def Find_Optimal_Aperture(self,souces, data):
        
        return True
    
    def Aperture(self,data, sources):
        positions = [sources['xcentroid'], sources['ycentroid']]
        #print(positions)
        aperture=phot.CircularAperture(positions, r=5.)
        table=phot.aperture_photometry(data, aperture)
        return table
    

    

class PhotometrySextractor():
    def __init__(self, global_parameters, tmp_dir):	
        self.test=True
    
    def MakeConfigFile(self, directory):
        config_file=directory+str(uuid.uuid4())+'.sex'
        return 1


class Photometry_Sextractor():
	def __init__(self, global_parameters, tmp_dir):	
		#self.file_name = file_name
		#self.ra=ra
		#self.dec=dec
		#self.radius=radius
		#self.degr=degr
		#self.fov_x=fov_x
		#self.fov_y=fov_y
		self.test=True
		self.SEXTRACTOR=global_parameters['SEXTRACTOR']
		self.BACK_SIZE=global_parameters['BACK_SIZE']
		self.BACK_FILTERSIZE=global_parameters['BACK_FILTERSIZE']
		self.DETECT_MINAREA=global_parameters['DETECT_MINAREA']
		self.DETECT_THRESH=global_parameters['DETECT_THRESH']
		self.DEBLEND_NTHRESH=global_parameters['DEBLEND_NTHRESH']
		self.DEBLEND_MINCONT=global_parameters['DEBLEND_MINCONT']
		self.REMOVE_FLAG = global_parameters['REMOVE_FLAG']
		self.FILTER_NAME=  global_parameters['FILTER_NAME']
		self.STARNNW_NAME=  global_parameters['STARNNW_NAME']
		self.CHECK_IMAGE=  global_parameters['SAVE_CHECK_IMAGES'].strip().upper()
		self.CHECK_IMAGE_TYPE = global_parameters['CHECKIMAGE_TYPE'].strip().upper()
		self.TMP_DIR=tmp_dir
		#self.VERBOSE=global_parameters['VERBOSE'].strip().upper()
		#self.
		
		#print(self.file_name)
		
	def MakeConfigFile(self, directory):
		
		config_file=directory+str(uuid.uuid4())+'.sex'
		measure_file=directory+str(uuid.uuid4())+'.param'
		f=open(measure_file,'w')
		f.write('\n')
		f.write('FLAGS\n')
		f.write('ALPHAWIN_J2000\n')  # coordinates alpha and delta
		f.write('DELTAWIN_J2000\n')
		f.write('XWIN_IMAGE\n')
		f.write('YWIN_IMAGE\n') 
		#f.write('FLUX_AUTO\n')
		#f.write('FLUXERR_AUTO\n')
		#f.write('FLUX_PSF\n')
		#f.write('FLUXERR_PSF\n')
		f.write('MAG_AUTO\n')
		f.write('MAGERR_AUTO\n')
		#f.write('MAG_PSF\n')
		#f.write('MAGERR_PSF\n')
	
		#f.write('FLUX_APER[12]\n')
		#f.write('FLUXERR_APER[12]\n')
		#f.write('MAG_APER[12]\n')
		#f.write('MAGERR_APER[12]\n')		
		f.write('ELLIPTICITY\n')	
		f.write('FWHM_IMAGE\n')
		f.write('SNR_WIN \n')
		##f.write('FWHMPSF_IMAGE \n')
		##
		f.close()
		
		f=open(config_file, "w")
		f.write('CATALOG_TYPE  ASCII_HEAD\n')
		f.write('PARAMETERS_NAME '+ measure_file+'\n')
		f.write('DETECT_TYPE	CCD\n')
		f.write('BACK_TYPE    Auto\n')
		f.write('BACK_SIZE  '+self.BACK_SIZE+'\n')
		f.write('BACK_FILTERSIZE '+self.BACK_FILTERSIZE+'\n')
		f.write('BACKPHOTO_TYPE  Global\n')
		f.write('DETECT_MINAREA  '+self.DETECT_MINAREA+'\n')
		f.write('DETECT_THRESH  '+self.DETECT_THRESH+'\n')
		f.write('ANALYSIS_THRESH  '+self.DETECT_THRESH+'\n')
		f.write('DEBLEND_NTHRESH  '+self.DEBLEND_NTHRESH+'\n')
		f.write('DEBLEND_MINCONT  '+self.DEBLEND_MINCONT+'\n')
		f.write('WEIGHT_TYPE BACKGROUND\n')
		f.write('CLEAN   Y\n')
		f.write('CLEAN_PARAM  1\n')
		f.write('MASK_TYPE  Correct'+'\n')
		f.write('FILTER  Y\n')
		f.write('FILTER_NAME '+self.FILTER_NAME+'\n')
		f.write('VERBOSE_TYPE     QUIET\n')  
		f.write('FLAG_TYPE     MAX\n')  
		f.write('MAG_ZEROPOINT  20\n')
		f.write('PHOT_AUTOPARAMS  2.5 3.5\n')
		f.write('MEMORY_OBJSTACK 65000\n')
		f.write('MEMORY_PIXSTACK 5000000\n')
		f.write('MEMORY_BUFSIZE 1024\n')

	#if global_param['USE_CCD_FITS'].lower().strip()=='y':
		#gain=image['Gain']
		#satur=image['Satur']
	#else:
		#gain=global_param['GAIN']
		#satur=global_param['SATUR']
		f.write('GAIN  1.5\n')
		f.write('SATUR_LEVEL 55000\n')
		f.write('PIXEL_SCALE  0\n')
		f.write('STARNNW_NAME      /usr/local/share/sextractor/default.nnw\n')
		if self.CHECK_IMAGE=='Y':
			check_name=directory+'check.fit'
			if self.CHECK_IMAGE_TYPE=='D':
				f.write('CHECKIMAGE_TYPE APERTURES\n')
				f.write('CHECKIMAGE_NAME '+ check_name+' \n')
			if self.CHECK_IMAGE_TYPE=='B':
				f.write('CHECKIMAGE_TYPE BACKGROUND\n')
				f.write('CHECKIMAGE_NAME '+ check_name+' \n')
			if self.CHECK_IMAGE_TYPE=='R':
				f.write('CHECKIMAGE_TYPE -BACKGROUND\n')
				f.write('CHECKIMAGE_NAME '+ check_name+' \n')
			
			#check_name=directory+'check1.fit'
			#f.write('CHECKIMAGE_TYPE BACKGROUND\n')
			#f.write('CHECKIMAGE_NAME '+ check_name+' \n')

		#check_name=tmp_dir+os.path.basename(image['WorkName']).split(".")[0].strip()+'.chk'
		#f.write('CHECKIMAGE_TYPE APERTURES\n')
		#f.write('CHECKIMAGE_NAME '+ check_name+' \n')
		#f.write('PHOT_APERTURES 4 6 8 10 12 14 16 18 20 22 25 30\n') 
		f.close()  		
		
		return config_file
	
	
	def Sextractor(self, file_name, config_name):

		catalog=os.path.basename(file_name).split(".")[0].strip()+'.cat'	
		catalog=self.TMP_DIR+catalog
		sextractor=self.SEXTRACTOR+' '+file_name+' -c '+config_name + ' -CATALOG_NAME '+catalog
		#print()
		#print(sextractor)
		#sys.exit()
		os.system(sextractor)
		if self.CHECK_IMAGE=='Y':
			if self.CHECK_IMAGE_TYPE=='D':
				check_image_name=os.path.basename(file_name).split(".")[0].strip()+'_aperture.fit'
				check_image_name=self.TMP_DIR+'/'+check_image_name
				os.rename(self.TMP_DIR+'/check.fit', check_image_name)
			if self.CHECK_IMAGE_TYPE=='B':
				check_image_name=os.path.basename(file_name).split(".")[0].strip()+'_backgr.fit'
				check_image_name=self.TMP_DIR+'/'+check_image_name
				os.rename(self.TMP_DIR+'/check.fit', check_image_name)
			if self.CHECK_IMAGE_TYPE=='R':
				check_image_name=os.path.basename(file_name).split(".")[0].strip()+'_backgr-removed.fit'
				check_image_name=self.TMP_DIR+'/'+check_image_name
				os.rename(self.TMP_DIR+'/check.fit', check_image_name)
		#check_image_name=os.path.basename(file_name).split(".")[0].strip()+'_backgr.fit'
		#check_image_name=os.path.dirname(file_name)+'/'+check_image_name
		#os.rename(os.path.dirname(file_name)+'/check1.fit', check_image_name)
		
		return catalog
	
	def Clear(self, table, flag=1):
		#print (table.colnames)
		rows=[]		
		#print('flag',flag)
		for row in table:
			#print(int(row['FLAGS']))
			if not int(row['FLAGS'])>flag:
				#print('in')
				rows.append(row)
				##new_table(rows=row)
				#print(row['FLAGS'])
		new_table=Table(rows=rows, names=table.colnames)
				
		return new_table
	
	def Mask(self, table, mask='B', box=(10,10,10,10), radius=500, mask_file=None):
		rows=[]
		naxis1=table.meta['Naxis1']
		naxis2=table.meta['Naxis2']
		
		
		
		if mask=='B' or mask=='BOX':
			# remove objects from the edges of image closer than boundary
			
			x1=box[0]
			y1=box[1]
			x2=box[2]
			y2=box[3]			
			
			for row in table:
				x=row['XWIN_IMAGE']
				y=row['YWIN_IMAGE']
				if x>=x1 and y>=y1 and x<=naxis1-x2 and y<=naxis2-y2:
					#print (x,y)
					rows.append(row)

				##new_table(rows=row)
				#print(row['FLAGS'])
			new_table=Table(rows=rows, names=table.colnames)
			new_table.meta=table.meta
			#new_table=table
		elif mask=='R' or mask=='CIRCLE':
			
			center1=int(naxis1/2)
			center2=int(naxis2/2)
			for row in table:
				x=row['XWIN_IMAGE']
				y=row['YWIN_IMAGE']
				distance=np.sqrt((x-center1)*(x-center1)+(y-center2)*(y-center2))
				if distance<=radius:
					rows.append(row)
					
			new_table=Table(rows=rows, names=table.colnames)	
			new_table.meta=table.meta
		
		return new_table
	
	def AddMeta(self, table, meta):
		
		#new_meta={'name': 'ff'}
		new_table=table
		new_table.meta=meta
		
		return new_table
	def AirMass(self, table):
		#rows=[]
		#for row in table:
		time=table.meta['Dateobs']
		latitude=table.meta['ObsLat']
		longitude=table.meta['ObsLon']
		location=EarthLocation(lat=latitude*u.deg, lon=longitude*u.deg, height=100*u.m)
		objects=SkyCoord(ra=table['ALPHAWIN_J2000']*u.degree, dec=table['DELTAWIN_J2000']*u.degree, frame='icrs')
		time=Time(time, format='isot', scale='utc')
		
		airmass=objects.transform_to(AltAz(obstime=time, location=location))
		airmass=np.array(airmass.secz)
		
		t=Table([airmass], names=('Airmass',))
		#print(t)
		t['Airmass'].format='%.5f'
		new_table=hstack([table, t])
		new_table.meta=table.meta
		
		return new_table
