import sqlite3, sys, os
from urllib.request import pathname2url
from astropy.wcs import WCS, utils
import numpy as np
from astropy.table import Table, hstack, Column
from astropy import time, coordinates as coord, units as u
import warnings
from astropy.utils.exceptions import AstropyWarning



KEYWORDS = ('VERBOSE','LOGGING','PROGRESS',
            'DEFAULT_PATH','CATALOG_PATH','DATABASE_REWRITE', 'DATABASE_NAME',
        'TESTING', 'ASTROMETRY', 'PHOTOMETRY','CALIBRATE', 'DATABASE',
        'API_KEY', 'SIMBAD_CENTRAL', 'RA_CENTRAL', 'DEC_CENTRAL',
        'BACKGROUND','SIGMA_CLIP', 'MEDIAN_FACTOR', 'MEAN_FACTOR', 'BOX_SIZE', 'FILTER_SIZE','MASK',
        'FWHM','TRESHOLD','GAIN','MINAREA',
        'SEXTRACTOR', 'BACK_SIZE', 'BACK_FILTERSIZE', 'DETECT_MINAREA', 'DETECT_THRESH', 'DEBLEND_NTHRESH','DEBLEND_MINCONT', 'REMOVE_FLAG', 'FILTER_NAME', 'STARNNW_NAME','MASK_TYPE','BOX','MASK_RADIUS',
	    'MAKE_ASTROMETRY','ASTROMETRY','RADIUS', 'DISTORSION',
	    'REDUCTION','NOTE', 'QUALITY', 'SAVE_CHECK_IMAGES', 'CHECKIMAGE_TYPE','SAVE_CALIB_IMAGES',
	    'CALC_HELCOR', 'CALC_BARCOR','COR_STARS', 'POS_DIFF',
	    'MIN_STARS', 'CALIB_POLY', 'SIGMA_CLIP','LIST_PARAM', 'PLOT_GRAPH', 'ATM_EXTINCTION',
	    'DIFF_AVERAGE_AUTO', 'MINIMUM_STARS_AUTO', 'COMP_STARS','COMP_STARS_AVERAGE', 'MEASURE_ALL', 'MEASURED_STARS',
	    'COMP_EXCLUDE','ROBUST_AVERAGE','MINIMUM_DATA_POINTS', 'COUNT_EXCLUDE', 
	    'DIFF_TYPE', 'DIFF_COMP_NAME', 'DIFF_MINIMUM_STARS', 'DIFF_BRIGHT_PERCENTAGE',
	    'VO_SAVE', 'VO_DB_REWRITE','VO_MIN_POINTS')


class ParameterFile():
	def __init__(self, parameter_file, keywords=KEYWORDS):	
		self.parameter_file=parameter_file
		self.KEYWORDS=KEYWORDS
        
	def Read(self):
	
		try:
			f = open(self.parameter_file)
			try:
				lines=f.readlines()
			#print "OK"
			finally:
				f.close
		except IOError:
			print()
			print()
			print( "ERROR - No such a file: "+self.parameter_file+ "!!")
			print('Pipeline is stopped')
			sys.exit()  
			
		global_parameters={}
		for line in lines:
			if not(line[0]=="#"):
				a=line.rstrip().split()
     
				if not len(a)==0:
					if a[0] in self.KEYWORDS:
						subline=line[len(a[0])+1:]
						try:
							subline.index('#')
							global_parameters[a[0]]=subline[:subline.index('#')].strip()	    
						except:
							global_parameters[a[0]]=subline
		return global_parameters
		
class Progress():
    #def __init__(self, name):	
     #   self.Name=name
        
    def Connect(name):
        conn=sqlite3.connect(name)
        cursor=conn.cursor()
        query  ='CREATE TABLE IF NOT EXISTS observation (jd PRIMARY KEY, filename, test, astrometry, sources, photometry, crosside, calibrate, database)'
        cursor.execute(query)
        return conn, cursor
    
    def Check_In_Database(db,cursor, jd):
        old_observations=[]
        #jd=float(image['JD'])
        query='SELECT jd FROM observation'
        cursor.execute(query)
        records=list(cursor.fetchall())
        old=[]
        for row in records:
            old.append(row[0])
        if jd in old:
            is_in=True
        else:
            is_in=False
       
        return is_in
        
    def Check_What_To_Do(db, cursor, jd):
        query='SELECT * FROM observation WHERE jd='+str(jd)
        cursor.execute(query)
        records=list(cursor.fetchall())
        #print(records[0])
        what_to_do={'test':bool(records[0][2]),'astrometry':bool(records[0][3]), 'sources':bool(records[0][4]),
                    'photometry':bool(records[0][5]), 'crossid':bool(records[0][6]), 'calibrate':bool(records[0][7])}
       
        return what_to_do
    
    def Write_To_Database(database, cursor, image):
        
        jd=image['JD']
        name=image['ORIGINAL_NAME']
        query='INSERT OR REPLACE INTO observation (jd, filename, test, astrometry, sources, photometry, crosside, calibrate, database) VALUES ('+str(jd)+',"'+name+'",0,1,1,1,1,1,1)' 
        cursor.execute(query)
        database.commit()
        return True
        
        
    def Update_Progress(database,cursor, jd, type='sources'):
        
        if type=='sources':       
            query='UPDATE observation SET sources=0 WHERE jd='+str(jd)
        if type=='photometry':            
            query='UPDATE observation SET photometry=0 WHERE jd='+str(jd)
        if type=='crossid':       
            query='UPDATE observation SET crosside=0 WHERE jd='+str(jd)
        if type=='calib':            
            query='UPDATE observation SET calibrate=0 WHERE jd='+str(jd)
        if type=='database':            
            query='UPDATE observation SET database=0 WHERE jd='+str(jd)  
        
        cursor.execute(query)
        database.commit()
        
        return True
    
     
 
def Convert_To_Table(xwin, ywin, flux, fluxerr, mag, magerror,flag):
    data=Table()
    data['XWIN']=xwin
    data['YWIN']=ywin
    data['FLUX_AUTO']=flux
    data['FLUXERR_AUTO']=fluxerr
    data['MAG_AUTO']=mag
    data['MAGERR_AUTO']=magerror
    data['FLAGS']=flag
    data['XWIN'].info.format='%.5f'
    data['YWIN'].info.format = '7.3f'
    
    return data   
 
def Convert_To_WORLD(image, measurements):
    w=WCS(image)
    x=measurements['XWIN']
    y=measurements['YWIN']
    pixcrd = np.column_stack((x,y))
    crd=w.all_pix2world(pixcrd, 0)
    
    return crd[:,0], crd[:,1]
    
def Clear(table, flag=1):
    rows=[]		
    for row in table:
        if not int(row['FLAGS'])>flag:
            rows.append(row)
    new_table=Table(rows=rows, names=table.colnames)
				
    return new_table

def Calculate_BJD(image):
    location=coord.EarthLocation.from_geodetic(image['OBSLAT'], image['OBSLON'])
    obs_time=time.Time(image['JD'], format='jd', scale='utc', location=location)
    
    alpha=image['CRVAL1']
    delta=image['CRVAL2']
    star=coord.SkyCoord(alpha, delta,unit=(u.deg, u.deg), frame='icrs')
    ltt_bary = obs_time.light_travel_time(star)
    tmp_times_bary=obs_time.tdb + ltt_bary
    bar_cor=(tmp_times_bary-obs_time).value
	    
    return bar_cor

	
def FOV(filename=None, header=None):
	if header is None:
		filename=filename
		hdu=fits.open(filename) 
		header=hdu[0].header
	else:
		header=header
	warnings.filterwarnings('ignore', category=AstropyWarning)	
	w = WCS(header)
	
	scale=utils.proj_plane_pixel_scales(w)
	pixel_scale=scale[0]
	
	fov1=header['NAXIS1']*scale[0]
	fov2=header['NAXIS2']*scale[1]
			
	return fov1, fov2, pixel_scale

def CENTRAL(filename=None, header=None):
	if header is None:
		filename=filename
		hdu=fits.open(filename) 
		header=hdu[0].header
	else:
		header=header
		
	warnings.filterwarnings('ignore', category=AstropyWarning)	
	w = WCS(header)
	
	centralX, centralY=w.all_pix2world(int(header['NAXIS1']/2), int(header['NAXIS2']/2), 0, ra_dec_order=True)
	
			
	return centralX, centralY
	
def Save_Name(path, name, type='back'):
    if type=='back':
        save_path=path+'/background/'
        save_name='bkg_'+os.path.basename(name)
        save_name=os.path.splitext(save_name)[0]+'.png'
    if type=='src':
        save_path=path+'/sources/'
        save_name='src_'+os.path.basename(name)
        save_name=os.path.splitext(save_name)[0]+'.tbl'
    if type=='src_plt':
        save_path=path+'/sources/'
        save_name='src_'+os.path.basename(name)
        save_name=os.path.splitext(save_name)[0]+'.png'
    if type=='phot':
        save_path=path+'/photometry/'
        save_name='phot_'+os.path.basename(name)
        save_name=os.path.splitext(save_name)[0]+'.tbl'    
    if type=='crossid':
        save_path=path+'/crossid/'
        save_name='cross_'+os.path.basename(name)
        save_name=os.path.splitext(save_name)[0]+'.tbl'  
    if type=='calib':
        save_path=path+'/calib/'
        save_name='calib_'+os.path.basename(name)
        save_name=os.path.splitext(save_name)[0]+'.tbl'
    if type=='fits':
        save_path=path+'/'
        save_name='res_'+os.path.basename(name)
        #save_name=os.path.splitext(save_name)[0]+'.tbl'
    
    if not os.path.isdir(save_path):
        os.makedirs(save_path)
    
    return save_path+save_name
    
def New_Commnets(table, zero_point, zero_point_error, number):
    table.meta={}
    table.meta['comments']=[str(round(zero_point,6)), str(round(zero_point_error,6)), str(number)]
    
    
    return table
           

class List():
	# create list of files for photometry 
	def __init__(self, directory, lst=None):	
		self.directory = directory
		self.lst=lst
		self.parameter={}
		
			
	
	def ListFiles(self):
		file_name=[]
		if self.lst is None:
			tmp=os.listdir(self.directory)
			onlyfiles=[f for f in tmp if os.path.isfile(os.path.join(self.directory,f))]
			for f in onlyfiles:
				if f.endswith(".fit") or f.endswith(".FIT") or f.endswith(".fts") or f.endswith(".FTS") or f.endswith(".fits") or f.endswith(".FITS"):
					file_name.append(os.path.join(self.directory, f))
						
				if f.endswith(".fit.gz") or f.endswith(".FIT.gz") or f.endswith(".fts.gz") or f.endswith(".FTS.gz") or f.endswith(".fits.gz") or f.endswith(".FITS.gz"):
					file_name.append(os.path.join(self.directory, f))
						
				if f.endswith(".fit.fz") or f.endswith(".FIT.fz") or f.endswith(".fts.fz") or f.endswith(".FTS.fz") or f.endswith(".fits.fz") or f.endswith(".FITS.fz"):
					file_name.append(os.path.join(self.directory, f)) 
			
			#file_name=onlyfiles
		else:
			f=open(self.lst,"r")
			tmplist=f.readlines()
			f.close()
			for f in tmplist:
				f=f.strip()
				if f.endswith(".fit") or f.endswith(".FIT") or f.endswith(".fts") or f.endswith(".FTS") or f.endswith(".fits") or f.endswith(".FITS"):
					file_name.append(f)
					
				if f.endswith(".fit.gz") or f.endswith(".FIT.gz") or f.endswith(".fts.gz") or f.endswith(".FTS.gz") or f.endswith(".fits.gz") or f.endswith(".FITS.gz"):
					file_name.append(f)			
					
				if f.endswith(".fit.fz") or f.endswith(".FIT.fz") or f.endswith(".fts.fz") or f.endswith(".FTS.fz") or f.endswith(".fits.fz") or f.endswith(".FITS.fz"):
					file_name.append(f)  
		#file_name=[]
		#if self.lst is None:
			#for top, dirs,  files  in os.walk(self.directory):
				#for f in files:
					#f=os.path.join(top, f)
					##print(f)
					#if f.endswith(".fit") or f.endswith(".FIT") or f.endswith(".fts") or f.endswith(".FTS") or f.endswith(".fits") or f.endswith(".FITS"):
						#file_name.append(os.path.join(top, f))
						
					#if f.endswith(".fit.gz") or f.endswith(".FIT.gz") or f.endswith(".fts.gz") or f.endswith(".FTS.gz") or f.endswith(".fits.gz") or f.endswith(".FITS.gz"):
						#file_name.append(os.path.join(top, f))
						
					#if f.endswith(".fit.fz") or f.endswith(".FIT.fz") or f.endswith(".fts.fz") or f.endswith(".FTS.fz") or f.endswith(".fits.fz") or f.endswith(".FITS.fz"):
						#file_name.append(f)  
						
		#else:
			#f=open(self.lst,"r")
			#tmplist=f.readlines()
			#f.close()
			#for f in tmplist:
				#f=f.strip()
				#if f.endswith(".fit") or f.endswith(".FIT") or f.endswith(".fts") or f.endswith(".FTS") or f.endswith(".fits") or f.endswith(".FITS"):
					#file_name.append(f)
					
				#if f.endswith(".fit.gz") or f.endswith(".FIT.gz") or f.endswith(".fts.gz") or f.endswith(".FTS.gz") or f.endswith(".fits.gz") or f.endswith(".FITS.gz"):
					#file_name.append(f)			
					
				#if f.endswith(".fit.fz") or f.endswith(".FIT.fz") or f.endswith(".fts.fz") or f.endswith(".FTS.fz") or f.endswith(".fits.fz") or f.endswith(".FITS.fz"):
					#file_name.append(f)  
					
		
		file_name.sort()
		
		return file_name 
	
	def Data(self, file_name):
		hdu=fits.open(file_name) 
		header=hdu[0].header
		data=hdu[0].data
		hdu.close()
		return header, data
	
	def ImageType(self, file_name, header=None):
		#parameter={}
		
		if header is None:
			hdu=fits.open(file_name) 
			header=hdu[0].header
		else:
			header=header
		tmp_imagetype_keyword='IMAGETYP'
		try:
			tmp_imtype=header[tmp_imagetype_keyword].strip().lower()				
			if tmp_imtype=='':
				imtype='Light'
			if tmp_imtype=='object' or tmp_imtype=='light' or tmp_imtype=='light frame' or tmp_imtype=='reduced_light':
				imtype='Light'
			if tmp_imtype=='flat' or tmp_imtype=='flat frame' or tmp_imtype=='flat-field' or tmp_imtype=='flat field' or tmp_imtype=='flat-field frame' or tmp_imtype=='flat field frame' or tmp_imtype=='flat_field':
				imtype='Flat'
			if tmp_imtype=='dark' or tmp_imtype=='dark frame' or tmp_imtype=='dark-frame' or tmp_imtype=='dark_frame':
					imtype='Dark'
			if tmp_imtype=='bias' or tmp_imtype=='bias frame' or tmp_imtype=='bias-frame' or tmp_imtype=='bias_frame':
				imtype='Bias'
			if tmp_imtype=='masterbias' or tmp_imtype=='masterbias frame' or tmp_imtype=='master bias frame' or tmp_imtype=='master bias' or tmp_imtype=='master_bias':
					imtype='MasterBias'
			if tmp_imtype=='masterdark' or tmp_imtype=='masterdark frame' or tmp_imtype=='master dark frame' or tmp_imtype=='master dark' or tmp_imtype=='master_dark' or tmp_imtype=='MasterDark':
				imtype='MasterDark'
			if tmp_imtype=='masterflat' or tmp_imtype=='masterflat frame' or tmp_imtype=='master flat frame' or tmp_imtype=='master flat' or tmp_imtype=='master_flat' or tmp_imtype=='MasterFlat':
					imtype='MasterFlat'
		except:
			imtype='Light'
		
		
		self.parameter['Image_Type']=imtype
		if header is None:
			hdu.close()
		return self.parameter
	
	def Parameters(self, file_name, header=None):
		self.parameter={}
		#print(file_name)
		#print
		#print
		
		if header is None:
			hdu=fits.open(file_name) 
			if  len(hdu)==1:
				header=hdu[0].header
				tb=1
			elif len(hdu)==2:
				header=hdu[0].header
				tb_header=hdu[1].header
				tb_data=hdu[1].data
				tb=2
			elif len(hdu)==3:
				header=hdu[0].header
				tb_header=hdu[1].header
				tb_data=hdu[1].data
				tb_header_un=hdu[2].header
				tb_data_un=hdu[2].data
				tb=3
			else:
				header=hdu[0].header
				tb=1
		else:
			header=header
		
		
		self.parameter=List.ImageType(self, file_name, header=header)
		
		naxis1=int(header['NAXIS1'])
		naxis2=int(header['NAXIS2'])
		
		wcs_keyword='CD1_1'
		try:
			tmp_wcs=header[wcs_keyword]
			wcs='Y'
		except:
			wcs='N'
			
		if wcs=='Y':
			crval1=header['CRVAL1']
			crval2=header['CRVAL2']
			crpix1=header['CRPIX1']
			crpix2=header['CRPIX2']
			fovX, fovY, pixel_scale=class_astrometry.FOV(header=header)
			centralX, centralY=class_astrometry.CENTRAL(header=header)
		else:
			crval1=0
			crval2=0
			crpix1=0
			crpix2=0
			fovX=0
			fovY=0
			centralX=0
			centralY=0
			pixel_scale=0
		
		tmp_exposure_keyword='EXPTIME'
		try:
			exposure=header[tmp_exposure_keyword]
		except:
			tmp_exposure_keyword='EXPOSURE'
			try:
				exposure=header[tmp_exposure_keyword]
			except:
				exposure=0
			
		try:
			dateobs=header['DATE-OBS']		
			if len(dateobs.strip())<11:
				timeobs=header['TIME-OBS']
			#print(timeobs)
				dateobs=dateobs+'T'+timeobs
			try:
				jd=header['JD']				
			except:
				t = Time(dateobs, format='isot', scale='utc')
				jd=t.jd
		except:
			dateobs=header['DATE-BEG']
			try:
				jd=header['JD']				
			except:
				t = Time(dateobs, format='isot', scale='utc')
				jd=t.jd+exposure/86400
		
		
		tmp_binning_keyword='BINNING'			
		try:
			xbinning=header[tmp_binning_keyword]
			ybinning=header[tmp_binning_keyword]				
		except:
			tmp_xbinning_keyword='XBINNING'
			tmp_ybinning_keyword='YBINNING'
			try:			
				xbinning=header[tmp_xbinning_keyword]
				ybinning=header[tmp_ybinning_keyword]			
			except:
				xbinning=1
				ybinning=1
		
		temp_temp_keyword='CCD-TEMP'
		try:
			temp=round(header[temp_temp_keyword])
		except:
			temp=0
		temp_filter_keyword='FILTER'
		try:
			imfilter=header[temp_filter_keyword]
		except:
			imfilter='None'	
		
		temp_telescope_keyword='TELESCOP'
		try:
			telescope=header[temp_telescope_keyword]
		except:
			telescope=''
		temp_instrument_keyword='INSTRUME'
		try:
			instrument=header[temp_instrument_keyword].strip()
		except:
			temp_instrument_keyword='DEVICE'
			try:
				instrument=header[temp_instrument_keyword].strip()
			except:
				instrument=''
		temp_observatory_keyword='OBSERVAT'
		try:
			observatory=header[temp_observatory_keyword].strip()
		except:
			temp_observatory_keyword='SITE'
			try:
				observatory=header[temp_observatory_keyword].strip()
			except:
				observatory='Kolonica'
				
		temp_obslat_keyword='SITELAT'
		try:
			obslat=header[temp_obslat_keyword].strip()
			tmp=obslat.split()
			obslat=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
			
		except:
			temp_obslat_keyword='OBS-LAT'
			try:
				obslat=header[temp_obslat_keyword].strip()
				tmp=obslat.split()
				obslat=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
				
			except:
				if observatory=='Kolonica':
					obslat=+48.934917
				else:
					obslat=0.0
		
		temp_obslong_keyword='SITELONG'
		try:
			obslon=header[temp_obslong_keyword].strip()
			tmp=obslon.split()
			obslon=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
			
		except:
			temp_obslong_keyword='OBS-LON'
			try:
				obslon=header[temp_obslong_keyword].strip()
				tmp=obslon.split()
				obslon=float(tmp[0])+float(tmp[1])/60 + float(tmp[2])/3600
				
			except:
				if observatory=='Kolonica':
					obslon=22.2738
				else:
					obslon=0.0
				
		temp_object_keyword='OBJECT'
		try:
			temp_object=header[temp_object_keyword].strip()
			if temp_object==' ' or temp_object=='' :
				temp_object='None'
		except:
			temp_object='None'
		
		temp_frame_keyword='FRAME'
		try:
			temp_frame=header[temp_frame_keyword].strip()
			if temp_frame==' ' or temp_frame=='' :
				temp_frame=''
		except:
			temp_frame=''
		# reading table data if exist
		
		
		self.parameter['Name']=file_name
		self.parameter['WCS']=wcs
		self.parameter['CRVAL1']=crval1
		self.parameter['CRVAL2']=crval2
		self.parameter['CRPIX1']=crpix1
		self.parameter['CRPIX2']=crpix2
		self.parameter['CENTER1']=centralX
		self.parameter['CENTER2']=centralY
		self.parameter['FOV1']=fovX
		self.parameter['FOV2']=fovY
		self.parameter['PIXSCALE']=pixel_scale
		self.parameter['Dateobs']=dateobs
		self.parameter['JD']=jd
		self.parameter['Naxis1']=naxis1
		self.parameter['Naxis2']=naxis2
		self.parameter['XBinning']=xbinning
		self.parameter['YBinning']=ybinning
		self.parameter['Temperature']=temp
		self.parameter['Filter']=imfilter
		self.parameter['ExpTime']=exposure
		self.parameter['Telescope']=telescope
		self.parameter['Camera']=instrument
		self.parameter['Observatory']=observatory
		self.parameter['ObsLat']=obslat
		self.parameter['ObsLon']=obslon
		self.parameter['Object']=temp_object
		self.parameter['Frame']=temp_frame
		if header is None:
			hdu.close()
		
		if tb==1:			
			self.parameter['Table']=''
			self.parameter['Table_Un']=''
		elif tb==2:
			tab=Table.read(file_name, hdu=1)
			self.parameter['Table']=tab
			self.parameter['Table_Un']=''
		elif tb==3:
			tab=Table.read(file_name, hdu=1)
			self.parameter['Table']=tab
			tab=Table.read(file_name, hdu=2)
			self.parameter['Table_Un']=tab
		return self.parameter
	
	def Compressed(self, name):
		if name.endswith(".fit") or name.endswith(".FIT") or name.endswith(".fts") or name.endswith(".FTS") or name.endswith(".fits") or name.endswith(".FITS"):
			compress_type='None'
		if name.endswith(".fit.gz") or name.endswith(".FIT.gz") or name.endswith(".fts.gz") or name.endswith(".FTS.gz") or name.endswith(".fits.gz") or name.endswith(".FITS.gz"):
			compress_type='gzip'
		if name.endswith('.fit.fz') or name.endswith('.FIT.fz') or name.endswith('.fts.fz') or name.endswith('.FTS.fz') or name.endswith('.fits.fz') or name.endswith('.FITS.fz'):
			compress_type='fpack'
				
		return compress_type
	
	def CopyToTmp(self, name_of_files, tmp_dir, OnlyLight=True,OnlyReduct=False):
		# name_of_files - list with file names in input directory or list
		# tmp_dir - path to temporary directory
		# OnlyLight - copy only light frames
		# OnlyReduct - copy only reduced frames
		# original_name - when copy use original name of the file
		
		
		tmp_file_names=[]
		
		number=1			
		name_of_files.sort()
		original_name=False
		file_list_parameters=[]
		
		for name in name_of_files:
			if OnlyLight:
				sys.stdout.write('\r\t- copy light images to temporary directory: '+str(number)+'/'+str(len(name_of_files)))
				sys.stdout.flush()
			else:
				sys.stdout.write('\r\t- copy bias, dark and flat images to temporary directory: '+str(number)+'/'+str(len(name_of_files)))
				sys.stdout.flush()
			number=number+1
		
			if List.Compressed(self, name)=='None':
				tmp_file_name=tmp_dir+os.path.basename(name)		
				pp=List.Parameters(self, name)		
				#print(pp)
				
				if OnlyLight :
					if pp['Image_Type'] == 'Light':
						if original_name:
							pass
						else:
							new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
							cp='cp "'+name+'" '+new_name
							os.system(cp)
							#shutil.copy2(name, new_name)
							pp['Name']=new_name
							file_list_parameters.append((new_name, pp))
				else:
					if not pp['Image_Type'] == 'Light':
						if original_name:
							pass
						else:	
							new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Image_Type'].replace(" ","")+'.fit'
							cp='cp "'+name+'" '+new_name
						#print(cp)
							os.system(cp)
						#shutil.copy2(name, new_name)
							pp['Name']=new_name
							file_list_parameters.append((new_name, pp))
					
			
			if List.Compressed(self, name)=='gzip':		
				tmp_file_name=tmp_dir+os.path.basename(name)					
				shutil.copy2(name, tmp_file_name)				
				os.system('gunzip -q '+tmp_file_name)		
				pp=List.Parameters(self, tmp_file_name[:-3])	
				
				if OnlyLight:
					if pp['Image_Type'] == 'Light':
						if original_name:
							pass
						else:
							new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
							os.rename(tmp_file_name[:-3], new_name)	
							pp['Name']=new_name
							file_list_parameters.append((new_name, pp))
					else:
						os.remove(tmp_file_name[:-3])
				else:
					if original_name:
						pass
					else:
						new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
						os.rename(tmp_file_name[:-3], new_name)	
						pp['Name']=new_name
						file_list_parameters.append((new_name, pp))
					
			if List.Compressed(self, name)=='fpack':
				tmp_file_name=tmp_dir+os.path.basename(name)					
				shutil.copy2(name, tmp_file_name)
				os.system('funpack -F '+tmp_file_name)	
				pp=List.Parameters(self, tmp_file_name[:-3])	
				
				if OnlyLight:
					if pp['Image_Type'] == 'Light':
						if original_name:
							pass
						else:
							new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
							os.rename(tmp_file_name[:-3], new_name)	
							pp['Name']=new_name	
							file_list_parameters.append((new_name, pp))
					else:
						os.remove(tmp_file_name[:-3])
				else:
					if original_name:
						pass
					else:
						new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
						os.rename(tmp_file_name[:-3], new_name)	
						pp['Name']=new_name
						file_list_parameters.append((new_name, pp))
					
					
		
		#for top, dirs,  files  in os.walk(tmp_dir):
			#for f in files:
				##parameters=List.Parameters(self, tmp_dir+f)
				#tmp_file_names.append(tmp_dir+f)  
				##tmp_file_names.append([tmp_dir+f, parameters])	

		return file_list_parameters
	#tmp_file_names
	
	def CopyDarkToTmp(self, name_of_files, tmp_dir):
		
		tmp_file_names=[]
		
		number=1			
		name_of_files.sort()
		
		for name in name_of_files:
			print(name)
			#sys.stdout.write('\rCopy images to temporary directory... '+str(number)+'/'+str(len(name_of_files)))
			#sys.stdout.flush()
			#number=number+1
		
			if name.endswith(".fit") or name.endswith(".FIT") or name.endswith(".fts") or name.endswith(".FTS") or name.endswith(".fits") or name.endswith(".FITS"):
				print(name)
				tmp_file_name=tmp_dir+os.path.basename(name)	
				pp=List.Parameters(self, name)	
				if pp['Image_Type'] == 'MasterDark':					
					shutil.copy2(name, tmp_file_name)
					
				
				#if pp['Image_Type'] == 'masterdark':
					#new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
					#os.rename(tmp_file_name, new_name)	
					#pp['Name']=new_name	
					##tmp_file_names.append(new_name)
					
				#else:
					#os.remove(tmp_file_name)								
					
			if name.endswith(".fit.gz") or name.endswith(".FIT.gz") or name.endswith(".fts.gz") or name.endswith(".FTS.gz") or name.endswith(".fits.gz") or name.endswith(".FITS.gz"):
					
				tmp_file_name=tmp_dir+os.path.basename(name)					
				shutil.copy2(name, tmp_file_name)
				os.system('gunzip -q '+tmp_file_name)		
				pp=List.Parameters(self, tmp_file_name[:-3])	
								
				if pp['Image_Type'] == 'Light':
					new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
					os.rename(tmp_file_name[:-3], new_name)	
					pp['Name']=new_name						
				else:
					os.remove(tmp_file_name[:-3])
					
					
			if name.endswith('.fit.fz') or name.endswith('.FIT.fz') or name.endswith('.fts.fz') or name.endswith('.FTS.fz') or name.endswith('.fits.fz') or name.endswith('.FITS.fz'):
				tmp_file_name=tmp_dir+os.path.basename(name)					
				shutil.copy2(name, tmp_file_name)
				os.system('funpack -F '+tmp_file_name)	
				pp=List.Parameters(self, tmp_file_name[:-3])	
								
				if pp['Image_Type'] == 'Light':
					new_name=tmp_dir+pp['Dateobs']+'_'+pp['Filter'].replace(" ","")+'_'+pp['Object'].replace(" ","")+'.fit'
					os.rename(tmp_file_name[:-3], new_name)	
					pp['Name']=new_name						
				else:
					os.remove(tmp_file_name[:-3])
					
		
		for top, dirs,  files  in os.walk(tmp_dir):
			for f in files:
				parameters=List.Parameters(self, tmp_dir+f)
				tmp_file_names.append(tmp_dir+f)
				#tmp_file_names.append([tmp_dir+f, parameters])	

		return tmp_file_names
	
	
	def Save_Master(self, name,median,parameters, MasterType=None, FlatType=None, Packed=None):
		if not MasterType==None:
			if os.path.isfile(name):
				os.remove(name)
			
			hdu=fits.PrimaryHDU(median)
			hdulist=fits.HDUList([hdu])
			hdu.scale('int16',bzero=32768)
			#hdu.scale('float32')
			hdulist.writeto(name)
			f=fits.open(name, mode='update')
			header=f[0].header
			header.append(fits.Card('XBINNING', parameters['XBinning'],''))
			header.append(fits.Card('YBINNING', parameters['YBinning'],''))
			header.append(fits.Card('DATE-OBS', parameters['Dateobs'][:10]+'T00:00:00',''))
			header.append(fits.Card('TELESCOP', parameters['Telescope'],''))
			header.append(fits.Card('INSTRUME', parameters['Camera'],''))
			header.append(fits.Card('SITE', parameters['Observatory'],''))
			header.append(fits.Card('SITELAT', parameters['ObsLat'],''))
			header.append(fits.Card('SITELONG', parameters['ObsLon'],''))
			if MasterType=='Flat':
				header.append(fits.Card('IMAGETYP', 'MasterFlat',''))
				header.append(fits.Card('FILTER', parameters['Filter'],''))
				if FlatType=='T':
					header['HISTORY'] ='Master flat created at '+str(datetime.date.today())+' as a median of twilight flats'
				else:
					header['HISTORY'] ='Master flat created at '+str(datetime.date.today())+' as a median of sky flats'
			if MasterType=='Dark':	
				header.append(fits.Card('IMAGETYP', 'MasterDark',''))
				header.append(fits.Card('EXPTIME', parameters['ExpTime'],''))	
				header.append(fits.Card('CCD-TEMP', parameters['Temperature'],''))
				header['HISTORY'] ='Master dark created at '+str(datetime.date.today())+' as a median of dark frames'
			if MasterType=='Bias':	
				header.append(fits.Card('IMAGETYP', 'MasterBias',''))
				header['HISTORY'] ='Master bias created at '+str(datetime.date.today())+' as a median of bias frames'
			f.flush()
			f.close()
			if Packed=='fpack':
				pack='fpack -D '+name
				os.system(pack)
			if Packed=='gzip':
				pack='gzip -q '+name
				os.system(pack)
				
		return name
	
	def SaveNewAstrometry(self, fits_name, save_name):
		dir=os.path.dirname(save_name)+'/'
		
		if not(os.path.isdir(dir)):
			os.makedirs(dir)
			
		mv='cp '+fits_name+' '+save_name
		os.system(mv)
				
		return save_name
	
	def SaveTable2FITS(self, fits_name, save_name, table ):
		if os.path.isfile(save_name):
			os.remove(save_name)
		#print()
		#print(save_name)
		try:
			save_name=save_name.replace('.new', '.fit')
		except:
			pass
		cp='cp "'+fits_name+'" '+save_name
		#print(cp)
		os.system(cp)
		
		#print(save_name)
		hdulist=fits.open(save_name)
		col1 = fits.Column(name='USNO-B1.0', format='15A', array=table['USNO-B1.0'])
		col2 = fits.Column(name='APASS', format='15A', array=table['APASS'])
		col3 = fits.Column(name='SIMBAD', format='15A', array=table['SIMBAD'])
		try:
			col4 = fits.Column(name='RA', format='E', array=table['RAJ2000'])
			col5 = fits.Column(name='DEC', format='E', array=table['DEJ2000'])
		except:
			col4 = fits.Column(name='RA', format='15A', array=table['RAJ2000'])
			col5 = fits.Column(name='DEC', format='15A', array=table['DEJ2000'])
		
		try:
			col6 = fits.Column(name='MAG_AUTO', format='E', array=table['MAG_AUTO'])
			col7 = fits.Column(name='MAGERR_AUTO', format='E', array=table['MAGERR_AUTO'])
		except:
			col6 = fits.Column(name='MAG_AUTO', format='15A', array=table['MAG_AUTO'])
			col7 = fits.Column(name='MAGERR_AUTO', format='15A', array=table['MAGERR_AUTO'])
		
		try:
			col8 = fits.Column(name='BarCor', format='E', array=table['BarCor'])
		except:
			col8 = fits.Column(name='BarCor', format='15A', array=table['BarCor'])
		
		try:
			col9 = fits.Column(name='Airmass', format='E', array=table['Airmass'])
		except:
			col9 = fits.Column(name='Airmass', format='15A', array=table['Airmass'])
			
		if table['MAG_DIFF'][0]=='nan':			
			col22 = fits.Column(name='MAG_DIFF', format='15A', array=table['MAG_DIFF'])
		else:
			col22 = fits.Column(name='MAG_DIFF', format='E', array=table['MAG_DIFF'])
		if table['MAGERR_DIFF'][0]=='nan':			
			col23 = fits.Column(name='MAGERR_DIFF', format='15A', array=table['MAGERR_DIFF'])
		else:
			col23 = fits.Column(name='MAGERR_DIFF', format='E', array=table['MAGERR_DIFF'])
		if table['COMP_AVER'][0]=='nan':
			col24 = fits.Column(name='COMP_AVER', format='15A', array=table['COMP_AVER'])
		else:
			col24 = fits.Column(name='COMP_AVER', format='E', array=table['COMP_AVER'])			
		if table['COMPERR_AVER'][0]=='nan':
			col25 = fits.Column(name='COMPERR_AVER', format='15A', array=table['COMPERR_AVER'])
		else:
			col25 = fits.Column(name='COMPERR_AVER', format='E', array=table['COMPERR_AVER'])
		if table['COMP_NUM'][0]=='nan':			
			col26 = fits.Column(name='NUM_COMP', format='15A', array=table['COMP_NUM'])
		else:
			col26 = fits.Column(name='NUM_COMP', format='E', array=table['COMP_NUM'])			
		if table['ZERO_POINT'][0]=='nan':			
			col27 = fits.Column(name='ZERO_POINT', format='15A', array=table['ZERO_POINT'])
		else:
			col27 = fits.Column(name='ZERO_POINT', format='E', array=table['ZERO_POINT'])
		col28 = fits.Column(name='DIFF_METHOD', format='A', array=table['DIFF_METHOD'])
			
		
		cols = fits.ColDefs([col1, col2, col3, col4, col5, col6, col7, col8, col9, col22, col23,col24, col25, col26, col27, col28 ])
		       # , ,  ])
		    
		       #col10, col11, col12, col13, col14, col15, col16, col17, col18, col19, col22, col23, col24, col25, col26, col27, col28 ])
		
		
		#print(cols)
		#sys.exit()
		
		tbhdu = fits.BinTableHDU.from_columns(cols, name='Photometry-Matched')
		hdulist.append(tbhdu)
		
		name='proc_'+os.path.basename(save_name)
		path=os.path.dirname(save_name)+'/'
		name=path+name
		#print(name)
		if os.path.isfile(name):
			os.remove(name)
		hdulist.writeto(name)
		os.remove(save_name)
				
		
		return True
	
	def SaveProcessed(self, fits_name, save_name,table):
		
		
		if os.path.isfile(save_name):
			os.remove(save_name)
		#print()
		#print(save_name)
		try:
			save_name=save_name.replace('.new', '.fit')
		except:
			pass
		cp='cp "'+fits_name+'" '+save_name
		#print(cp)
		os.system(cp)
		
		hdulist=fits.open(save_name)
		hdulist[0].header.append(('Frame', table.meta['Frame']))
		#fits.setval(save_name, 'Frame', value=table.meta['Frame'])
		
		# matched table
		
		col1 = fits.Column(name='USNO-B1.0', format='15A', array=table['USNO-B1.0'])
		col2 = fits.Column(name='APASS', format='15A', array=table['APASS'])
		col3 = fits.Column(name='SIMBAD', format='15A', array=table['SIMBAD'])
		
		col4 = fits.Column(name='RA', format='E', array=table['RAJ2000'])
		col5 = fits.Column(name='DEC', format='E', array=table['DEJ2000'])
		col6 = fits.Column(name='MAG_AUTO', format='E', array=table['MAG_AUTO'])
		col7 = fits.Column(name='MAGERR_AUTO', format='E', array=table['MAGERR_AUTO'])
		col8 = fits.Column(name='BarCor', format='E', array=table['BarCor'])
		col9 = fits.Column(name='Airmass', format='E', array=table['Airmass'])
		#try:
			#col10 = fits.Column(name='Vmag', format='E', array=table['Vmag'])
			#col11 = fits.Column(name='e_Vmag', format='E', array=table['e_Vmag'])
		#except:
			#col10 = fits.Column(name='Vmag', format='2A', array=['--']*len(table['Vmag']))
			#col11 = fits.Column(name='e_Vmag', format='2A', array=['--']*len(table['Vmag']))
		#try:
			#col12 = fits.Column(name='Bmag', format='E', array=table['Bmag'])
			#col13 = fits.Column(name='e_Bmag', format='E', array=table['e_Bmag'])
		#except:
			#col12 = fits.Column(name='Bmag', format='2A', array=['--']*len(table['Vmag']))
			#col13 = fits.Column(name='e_Bmag', format='2A', array=['--']*len(table['Vmag']))
		#try:
			#col14 = fits.Column(name='g_mag', format='E', array=table['g_mag'])
			#col15 = fits.Column(name='e_g_mag', format='E', array=table['e_g_mag'])
		#except:
			#col14 = fits.Column(name='g_mag', format='2A', array=['--']*len(table['Vmag']))
			#col15 = fits.Column(name='e_g_mag', format='2A', array=['--']*len(table['Vmag']))
		#try:
			#col16 = fits.Column(name='r_mag', format='E', array=table['r_mag'])
			#col17 = fits.Column(name='e_r_mag', format='E', array=table['e_r_mag'])
		#except:
			#col16 = fits.Column(name='r_mag', format='2A', array=['--']*len(table['Vmag']))
			#col17 = fits.Column(name='e_r_mag', format='2A', array=['--']*len(table['Vmag']))
		#try:
			#col18 = fits.Column(name='i_mag', format='E', array=table['i_mag'])
			#col19 = fits.Column(name='e_i_mag', format='E', array=table['e_i_mag'])
		#except:
			#col18 = fits.Column(name='i_mag', format='2A', array=['--']*len(table['Vmag']))
			#col19 = fits.Column(name='e_i_mag', format='2A', array=['--']*len(table['Vmag']))
		
		#col20 = fits.Column(name='MAG_CALIB', format='E', array=table['MAG_CALIB'])
		#col21 = fits.Column(name='MAGERR_CALIB', format='E', array=table['MAGERR_CALIB'])
		#print(table['MAGERR_DIFF'][0])
		if table['MAG_DIFF'][0]=='nan':
			col22 = fits.Column(name='MAG_DIFF', format='I5A', array=table['MAG_DIFF'])
			col23 = fits.Column(name='MAGERR_DIFF', format='I5A', array=table['MAGERR_DIFF'])
		
			col24 = fits.Column(name='COMP_AVER', format='I5A', array=table['COMP_AVER'])
			col25 = fits.Column(name='COMPERR_AVER', format='I5A', array=table['COMPERR_AVER'])
			col26 = fits.Column(name='NUM_COMP', format='I5A', array=table['COMP_NUM'])
			col27 = fits.Column(name='ZERO_POINT', format='I5A', array=table['ZERO_POINT'])
			col28 = fits.Column(name='DIFF_METHOD', format='A', array=table['DIFF_METHOD'])
			
		else:
			col22 = fits.Column(name='MAG_DIFF', format='E', array=table['MAG_DIFF'])
			col23 = fits.Column(name='MAGERR_DIFF', format='E', array=table['MAGERR_DIFF'])
		
			col24 = fits.Column(name='COMP_AVER', format='E', array=table['COMP_AVER'])
			col25 = fits.Column(name='COMPERR_AVER', format='E', array=table['COMPERR_AVER'])
			col26 = fits.Column(name='NUM_COMP', format='E', array=table['COMP_NUM'])
			col27 = fits.Column(name='ZERO_POINT', format='E', array=table['ZERO_POINT'])
			col28 = fits.Column(name='DIFF_METHOD', format='A', array=table['DIFF_METHOD'])
			
		#col26= fits.Column(name='Var', format='2A', array=table['Var'])
		
		
		
		cols = fits.ColDefs([col1, col2, col3, col4, col5, col6, col7, col8, col9, col10, col11, col12, col13, col14, col15, col16, col17, col18, col19, col22, col23, col24, col25, col26, col27, col28 ])
		tbhdu = fits.BinTableHDU.from_columns(cols, name='Photometry-Matched')
		hdulist.append(tbhdu)
		#print(len(hdulist))
		
		#non matched table
		 
		#col1 = fits.Column(name='USNO-B1.0', format='10A', array=table_un['USNO-B1.0'])
		#col2 = fits.Column(name='RA', format='E', array=table_un['ALPHAWIN_J2000'])
		#col3 = fits.Column(name='DEC', format='E', array=table_un['DELTAWIN_J2000'])
		#col4 = fits.Column(name='MAG_AUTO', format='E', array=table_un['MAG_AUTO'])
		#col5 = fits.Column(name='MAGERR_AUTO', format='E', array=table_un['MAGERR_AUTO'])
		#col6 = fits.Column(name='BarCor', format='E', array=table_un['BarCor'])
		#col7 = fits.Column(name='Airmass', format='E', array=table_un['Airmass'])
		#col8 = fits.Column(name='J', format='E', array=table_un['J'])
		#col9 = fits.Column(name='J_ERR', format='E', array=table_un['J_ERR'])
		#col10 = fits.Column(name='H', format='E', array=table_un['H'])
		#col11 = fits.Column(name='H_ERR', format='E', array=table_un['H_ERR'])
		#col12 = fits.Column(name='K', format='E', array=table_un['K'])
		#col13  = fits.Column(name='K_ERR', format='E', array=table_un['K_ERR'])
		#col14 = fits.Column(name='B', format='E', array=table_un['B'])
		#col15 = fits.Column(name='B_ERR', format='E', array=table_un['B_ERR'])
		#col16 = fits.Column(name='V', format='E', array=table_un['V'])
		#col17 = fits.Column(name='V_ERR', format='E', array=table_un['V_ERR'])
		#col18 = fits.Column(name='g', format='E', array=table_un['g'])
		#col19 = fits.Column(name='g_ERR', format='E', array=table_un['g_ERR'])
		#col20 = fits.Column(name='r', format='E', array=table_un['r'])
		#col21 = fits.Column(name='r_ERR', format='E', array=table_un['r_ERR'])
		#col22 = fits.Column(name='i', format='E', array=table_un['i'])
		#col23 = fits.Column(name='i_ERR', format='E', array=table_un['i_ERR'])
		#col24 = fits.Column(name='Var', format='1A', array=table['Var'])
		#cols = fits.ColDefs([col1, col2, col3, col4, col5, col6, col7])
		#tbhdu = fits.BinTableHDU.from_columns(cols, name='Photometry-UnMatched')
		#hdulist.append(tbhdu)
		
		
		name='proc_'+os.path.basename(save_name)
		path=os.path.dirname(save_name)+'/'
		name=path+name
		#print(name)
		if os.path.isfile(name):
			os.remove(name)
		hdulist.writeto(name)
		os.remove(save_name)
				
		return name
	
	def SaveUnProcessed(self, fits_name, save_name):
		if os.path.isfile(save_name):
			os.remove(save_name)
		cp='cp "'+fits_name+'" '+save_name
		name='proc_'+os.path.basename(save_name)
		path=os.path.dirname(save_name)+'/'
		name=path+name
		#cp='cp "'+fits_name+'" '+name
		#print(cp)
		os.system(cp)
	
		return na
